#-*- coding: utf-8 -*-

BASE_URL = 'https://api.bge.genomics.cn'

# 支持的 OAuth2 授权方式
GRANT_AUTHORIZATION_CODE = 'authorization_code'
GRANT_TYPE_CREDENTIALS = 'client_credentials'

HTTP_DEFAULT_TIMEOUT = 16
HTTP_MAX_RETRIES = 3